"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KnapsackHbsRenderer = void 0;
const app_1 = require("@knapsack/app");
const file_utils_1 = require("@knapsack/file-utils");
const path_1 = require("path");
const handlebars_1 = __importDefault(require("handlebars"));
const isEmpty_1 = __importDefault(require("lodash/isEmpty"));
const utils_1 = require("./utils");
const { pkg } = (0, file_utils_1.findUpPkgJson)(__dirname);
const gitRoot = (0, file_utils_1.findGitRoot)();
class KnapsackHbsRenderer extends app_1.KnapsackRendererBase {
    constructor({ alterHandlebars = () => { }, } = {}) {
        super({
            id: 'hbs',
            extension: 'hbs',
            language: 'hbs',
        });
        this.getMeta = () => ({
            id: this.id,
            title: 'Handlebars',
            aliasUse: 'off',
            aliasTitle: '',
            aliasDescription: '',
            enableDataDemos: true,
            enableTemplateDemos: true,
            hasSlotsSupport: true,
            version: pkg.version,
            hasInferSpecSupport: false,
            hasTemplateSuggestionsSupport: true,
            prototypingTemplate: {
                path: '@knapsack/renderer-hbs/prototype-template.hbs',
                spec: {
                    isInferred: false,
                    props: {
                        type: 'object',
                        properties: {},
                    },
                    slots: {
                        content: { title: 'Content' },
                    },
                },
            },
        });
        // Wipe out all handlebars partials and vars
        this.handlebars = handlebars_1.default.create();
        this.alterHandlebars = alterHandlebars;
        this.alterHandlebars(this.handlebars);
    }
    async registerComponentPaths() {
        // Wipe out handlebars instance entirely and reregister everything
        this.handlebars = handlebars_1.default.create();
        const { patterns, allTemplates } = this.getMyTemplates();
        await Promise.all(allTemplates.map(async ({ templateId, patternId }) => {
            const { absolutePath, originalPath } = this.patterns.getTemplatePath({
                patternId,
                templateId,
            });
            // ie /path/to/file/ks-test-hbs/src/button.hbs
            const contents = await (0, file_utils_1.readFile)(absolutePath);
            try {
                // ie @knapsack-labs/ks-test-hbs/src/button.hbs
                this.handlebars.registerPartial(originalPath, contents);
            }
            catch (e) {
                const msg = `Couldn't register Handlebars partial. templateID: ${templateId}, patthernID: ${patternId}. Original error message: ${e.message}`;
                app_1.log.error(msg);
                throw e;
            }
            app_1.log.verbose(`Registered partial: templateId: ${templateId}, patternId: ${patternId}: ${originalPath}`);
        }));
        const protoContents = await (0, file_utils_1.readFile)(require.resolve(this.getMeta().prototypingTemplate.path));
        this.handlebars.registerPartial(this.getMeta().prototypingTemplate.alias, protoContents);
        // Running this AFTER we register all the Knapsack partials
        this.alterHandlebars(this.handlebars);
    }
    async init(options) {
        await super.init(options);
        await this.registerComponentPaths();
    }
    onChange() {
        super.onChange();
        this.registerComponentPaths();
    }
    async render(opt) {
        try {
            const usage = await this.getUsage(opt);
            let data = {};
            if (opt.demo.type === 'data') {
                const { globalData } = (0, utils_1.splitData)(opt.demo.data.props);
                data = globalData;
            }
            const template = this.handlebars.compile(usage);
            const html = template(data);
            return {
                ok: true,
                html,
                /**
                 * since we can only show one code block for usage and HBS rendering involves global data (i.e. handlebars context, the `data` var here) AND a template (the `usage` var) we're compromising by showing the data object above the template as that's the best we can do for now
                 */
                usage: Object.keys(data).length
                    ? // eslint-disable-next-line prefer-template
                        JSON.stringify(data, null, '  ') + '\n' + usage
                    : usage,
                templateLanguage: this.language,
            };
        }
        catch (err) {
            app_1.log.error(err);
            return {
                ok: false,
                message: err.message,
                html: `<p>Error rendering: ${err.message}</p>`,
            };
        }
    }
    async getUsage({ pattern, patternManifest, template, demo, }) {
        switch (demo === null || demo === void 0 ? void 0 : demo.type) {
            case 'template': {
                const { absolutePath, exists } = patternManifest.getTemplatePath({
                    patternId: pattern.id,
                    templateId: template.id,
                    demoId: demo.id,
                });
                if (!exists) {
                    throw new Error(`The Handlebars template demo does not exist at path ${absolutePath}`);
                }
                return (0, file_utils_1.readFile)(absolutePath);
            }
            case 'data': {
                const { data: { props, slots }, } = demo;
                // No slots from the UI (empty slots object)
                if ((0, isEmpty_1.default)(slots)) {
                    const usage = await (0, utils_1.getHbsUsage)({
                        templateName: template.path,
                        data: props,
                    });
                    return usage;
                }
                // We gots slots
                {
                    const slotIncludes = await Promise.all(Object.keys(slots).map(async (slotName) => {
                        const slottedTemplates = slots[slotName];
                        const slotContent = await Promise.all(slottedTemplates.map(async (slottedTemplate) => {
                            if (app_1.KnapsackRendererBase.isSlottedText(slottedTemplate)) {
                                return `  ${slottedTemplate.text}`;
                            }
                            if (app_1.KnapsackRendererBase.isSlottedTemplateReference(slottedTemplate)) {
                                throw new Error(`Cannot handle slotted template references in ${JSON.stringify({
                                    patternId: pattern.id,
                                    templateId: template.id,
                                    demoId: demo.id,
                                    slottedTemplate,
                                })}`);
                            }
                            const thisPattern = patternManifest.getPattern(slottedTemplate.patternId);
                            const thisTemplate = thisPattern.templates.find((t) => t.id === slottedTemplate.templateId);
                            const usage = await this.getUsage({
                                pattern: thisPattern,
                                template: thisTemplate,
                                patternManifest,
                                demo: slottedTemplate.demo ||
                                    this.patterns.demosById[slottedTemplate.demoId],
                            });
                            return usage;
                        }));
                        return {
                            name: slotName,
                            content: slotContent,
                        };
                    }));
                    // @TODO: need slot names?
                    // const availableSlotNames = Object.keys(template.spec.slots);
                    const usage = await (0, utils_1.getHbsUsage)({
                        templateName: template.path,
                        data: props,
                        slots: slotIncludes,
                    });
                    return usage;
                }
            }
            default:
                throw new Error(`Must pass in a demo.type`);
        }
    }
    async getTemplateSuggestions() {
        const { unusedPaths } = await this.getUnusedTemplatePaths({
            globPaths: [(0, path_1.join)(gitRoot, '**/*.{hbs,handlebars}')],
            cwd: gitRoot,
        });
        const suggestions = unusedPaths.map((path) => ({
            path,
        }));
        return { suggestions };
    }
}
exports.KnapsackHbsRenderer = KnapsackHbsRenderer;
//# sourceMappingURL=renderer-hbs.js.map